;
; (C) 2007 Marek Wodzinski http://majek.mamy.to/
;
;
;    ********************************************************************
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;    *********************************************************************
;
; TODO:
;	- comments only in one language:-)
;	- maybe exp instead of dual rates?
;
; Changelog:
; 2007.07.03	- first working release
; 2007.07.04	- ppm input moved to separate procedure
;		- start of calibration code
; 2007.07.08	- dual rates for elevator
; 2007.09.03	- switch for switching off dual rates
; 2007.11.27	- license, commnets, very small updates

.nolist
.include "tn13def.inc"

; ******** STAE *********
.equ	ZEGAR=9600000	;9.6MHz
.equ	NEUTRAL=1800	;1.5ms


; ***** REJESTRY *****
.def		zero=r2		;stala 0
.def		efef=r3		;stala 255
.def		isreg=r4	;tymczasowy na SREG do przerwan
.def		neutrall=r6	;neutral position
.def		neutralh=r7
.def		chAl=r8		;channel A
.def		chAh=r9
.def		chBl=r10	;channel B
.def		chBh=r11
.def		temp=r16	;zwykly rejestr tymczasowy
.def		temp2=r17	;drugi temp
.def		status=r18	;statusy
.def		timerh=r19	;starszy bajt licznika czasu
.def		itemp=r20	;jw. ale do wykorzystania w przerwaniach
.def		itemp2=r21	;drugi temp dla przerwa
.def		temp3=r22	;trzeci
.def		timeout=r23	;licznik do timeoutu wykrywania braku sygnalu
.def		tstartl=r24	;timer start
.def		tstarth=r25

; r26	XL		;wskaznik bufora (z rs-a)
; r27	XH
; r28	YL		;wskaznik bufora (do zapisu)
; r29	YH
; r30	ZL	- lpm
; r31	ZH

; ******** HARDWARE ********

; wejcia/wyjcia ppm
.equ		PPM_PORT=PORTB
.equ		PPM_DDR=DDRB
.equ		PPM_PIN=PINB
.equ		PPM_IN1=PORTB0
.equ		PPM_IN2=PORTB1
.equ		PPM_OUT1=PORTB3
.equ		PPM_OUT2=PORTB4
.equ		PPM_SWITCH=PORTB2

; bity w rejestrze statusowym
.equ		PPM_OV=0	;za dlugi impuls



; ###### makra ######
; #


;****Source code***************************************************
.list
.cseg					;CODE segment
.org 0
		rjmp	reset	;RESET
		reti		;INT0
		reti		;PCINT0
		rjmp	t0ov	;Timer0 Overflow
		reti		;EEPROM ready
		reti		;Analog comparator
		reti		;Timer0 CompareA
		reti		;Timer0 CompareB
		reti		;WDT
		reti		;ADC conversion complete

; start main program
reset:
		cli
		
		;watchdog init
		wdr
		in	temp,WDTCR
		ori	temp,(1<<WDCE) | (1<<WDE)
		out	WDTCR,temp
		ldi	temp,(1<<WDE) | (1<<WDP2) | (1<<WDP1)
		out	WDTCR,temp
		
		; set clock prescaler to 1, CLK=9.6MHz
		ldi	temp,(1<<CLKPCE)
		ldi	temp2,0
		out	CLKPR,temp
		out	CLKPR,temp2
		nop	;just in case:-)
		
		; Set stack pointer to last internal RAM location
		ldi	temp,low(RAMEND)
		out	SPL,temp

		;stae
		clr	zero
		ser	temp
		mov	efef,temp
		
		;rejestry
		mov	status,zero
		mov	timeout,zero

		;konfig z eeproma
		;ldi	temp,ee_neutral
		;out	EEARL,temp
		;sbi	EECR,EERE
		;in	neutrall,EEDR
		;inc	temp
		;out	EEARL,temp
		;sbi	EECR,EERE
		;in	neutralh,EEDR
		
		;inizjalizacja pozycji neutralnej
		ldi	temp,low(NEUTRAL)
		mov	neutrall,temp
		ldi	temp,high(NEUTRAL)
		mov	neutralh,temp
		
		;zmienne trzymajce aktualne wartoci kanaw
		movw	chAl,neutrall
		movw	chBl,neutrall
		
		;pin configuration
		cbi	PPM_DDR,PPM_IN1		;input1
		cbi	PPM_PORT,PPM_IN1	;no pull up
		
		cbi	PPM_DDR,PPM_IN2		;input2
		cbi	PPM_PORT,PPM_IN2
		
		sbi	PPM_DDR,PPM_OUT1	;output1
		cbi	PPM_PORT,PPM_OUT1	;low level standby
		
		sbi	PPM_DDR,PPM_OUT2	;output2
		cbi	PPM_PORT,PPM_OUT2
		
		cbi	PPM_DDR,PPM_SWITCH	;switch
		sbi	PPM_PORT,PPM_SWITCH	;pull-up
		

		;timer0
		ldi	temp,0b00000000	;COMA=00 (normal),COMB=00 (normal),WGM10=0 (normal)
		out	TCCR0A,temp
		ldi	temp,0b00000010	;prescaler 8 (F=1.2MHz)
		out	TCCR0B,temp
		ldi	temp,(1<<TOIE0)	;przerwanie od timer0 overflow
		out	TIMSK0,temp
		
		;calibration
		;mov	temp,neutrall
		;and	temp,neutralh
		;cpi	temp,255
		;brne	k0
		;rjmp	calibration
k0:
		
		;odblokowanie przerwa
		sei
		wdr
		
;glowna petla programu
kolko:	
		;get ppm pulses
		rcall	ppm_get	


		;calculate values and send pulses
		
		;dual rates
		movw	temp,chBl
		sbic	PPM_PORT,PPM_SWITCH	;check switch for dual rates
		rcall	dual_rates
		movw	chBl,temp
		
c1:		;OUT1=(A+B)/2
		movw	temp,chAl
		add 	temp,chBl
		adc	temp2,chBh
		clc
		ror	temp2
		ror	temp
		
		sbi	PPM_PORT,PPM_OUT1
		rcall	timer_wait
		cbi	PPM_PORT,PPM_OUT1
		
		
		;OUT2=(2*neutral+A-B)/2
		movw	temp,neutrall
		add	temp,temp
		adc	temp2,temp2
		add	temp,chAl
		adc	temp2,chAh
		sub	temp,chBl
		sbc	temp2,chBh
		clc
		ror	temp2
		ror	temp
		
		sbi	PPM_PORT,PPM_OUT2
		rcall	timer_wait
		cbi	PPM_PORT,PPM_OUT2
		
		wdr
		
		rjmp	kolko		




; ################  obsluga przerwan ##############
; #
; Dozwolone operacje na rejestrach bez ich zachowywania:
; - itemp


;
; # timer0 overflow: co ok. 0.23ms
t0ov:
		in	isreg,SREG	;zachowaj flagi

		inc	timerh		;odliczanie czasu
		brne	t0ov_e
		ori	status,(1<<PPM_OV)	;54ms - timeout
		
t0ov_e:
		out	SREG,isreg
		reti
;


;
;
; ############## koniec przerwa ###############



; ############## podprogramy ################
; #

;
;
timer_reset:
		out	TCNT0,zero
		mov	timerh,zero
		andi	status,(0xff-(1<<PPM_OV))
		ret
;


;
; temp2:temp - wait time
timer_wait:
		rcall	timer_reset
tw1:
		cp	timerh,temp2	;wait for high byte equal or greater
		brcs	tw1
tw2:
		in	temp3,TCNT0
		cp	temp3,temp
		brcs	tw2
		
		ret
;


;
; temp2:temp - time
check_range:
		;timer overflow
		sbrc	status,PPM_OV
		rjmp	cr_nok
		
		;value to small
		cpi	temp2,4
		brcs	cr_nok
		
		;value to large
		cpi	temp2,10
		brcc	cr_nok
		
		clc	;ok
		ret

cr_nok:
		sec
		ret
;


;
;
ppm_get:
		;wait for first impulse
k1:
		sbis	PPM_PIN,PPM_IN1	;rising edge
		rjmp	k1
		rcall	timer_reset
k2:
		sbic	PPM_PIN,PPM_IN1	;falling edge
		rjmp	k2
		mov	temp2,timerh	;get timer
		in	temp,TCNT0
		
		rcall	check_range
		brcs	k3		;if out of range, do not update value
		movw	chAl,temp	;save value
		
		;wait for second impulse
k3:
		sbis	PPM_PIN,PPM_IN2
		rjmp	k3
		rcall	timer_reset
k4:
		sbic	PPM_PIN,PPM_IN2
		rjmp	k4
		mov	temp2,timerh
		in	temp,TCNT0
		
		rcall	check_range
		brcs	k5
		movw	chBl,temp
k5:
		ret
;


;
;
dual_rates:
		add	temp,neutrall
		adc	temp2,neutralh
		clc
		ror	temp2
		ror	temp
		ret
;


;
;
calibration:
		
		rjmp	reset
;

; ############ dane ############
;

;segment danych		
.dseg

;*****Dane do eeproma******************************
.eseg					;EEPROM segment
ee_neutral:	.db	low(NEUTRAL)	;calibration: 1.5ms (1800 timer clocks at 1.2MHz)
		.db	high(NEUTRAL)
